﻿package controller
{
import flash.events.Event;
import flash.events.EventDispatcher;

import model.BasicData;
import model.NewsSourceData;
import model.SoundData;

import org.si.sion.SiONDriver;
import org.si.sion.SiONVoice;
import org.si.sion.events.SiONTrackEvent;
import org.si.sion.sequencer.SiMMLTrack;
import org.si.sound.DrumMachine;

import view_controller.Main3DViewController;

public class SoundController extends EventDispatcher
{
    private var _bdW:int = BasicData.TEXT_DISPLAY_WIDTH;
    private var _bdH:int = BasicData.TEXT_DISPLAY_HEIGHT;

    public var _driver:SiONDriver = new SiONDriver();
    public var _voices:Vector.<SiONVoice>;

    public var _length:Vector.<int>;
    public var _vol:Vector.<int>;

    private var drumMachine:DrumMachine;
    private var _trackUpper:SiMMLTrack;

    private var _newsSourceData:NewsSourceData;
    private var _main3DViewController:Main3DViewController;
    private var _soundData:SoundData;
    private var _noteOnPrevList:Array = [];

    public function SoundController(newsSourceData:NewsSourceData, soundData:SoundData, main3DViewController:Main3DViewController):void
    {
        _newsSourceData = newsSourceData;
        _soundData = soundData;
        _main3DViewController = main3DViewController;

        soundData.masterVolume = 0;

        init();
    }

    private function init():void
    {
        _voices = _soundData.voices;
        _length = _soundData.length;
        _vol = _soundData.vol;

        _driver.setBeatCallbackInterval(1);
        _driver.addEventListener(SiONTrackEvent.BEAT, _beatHandler);
        _driver.setTimerInterruption(1, _timerInterruptionHandler);
        _driver.bpm = _soundData.bpm;
        _driver.volume = SoundData.DRIVER_VOLUME;
        drumMachine = new DrumMachine();
        drumMachine.volume = SoundData.DRUM_VOLUME;

        for(var i:int = 0; i < _bdH; i++)
        {
            _noteOnPrevList[i] = 0;
        }

        _soundData.addEventListener(Event.CHANGE, soundDataChangeHandler);
        _soundData.addEventListener(SoundData.DRIVER_PLAY_CHANGE, driverPlayChangeHandler);
        _newsSourceData.addEventListener(NewsSourceData.SET_CURRENT_TEXT, setCurrentTextHandler);
    }

    private function driverPlayChangeHandler(event:Event):void
    {
        if(_soundData.isDriverPlay) driverPlay();
        else driverStop();
    }

    private function setCurrentTextHandler(event:Event):void
    {
        _soundData.notesKey = _newsSourceData.currentNewsData.categoryData.soundKey;
        _soundData.bpm = _newsSourceData.currentNewsData.categoryData.bpm;
        _soundData.setVoice(_newsSourceData.currentNewsData.categoryData.voice);
    }

    private function driverPlay():void
    {
        _driver.play();
        drumMachine.play();
    }
    private function driverStop():void
    {
        _driver.stop();
        drumMachine.stop();
    }

    private function soundDataChangeHandler(event:Event):void
    {
        _driver.bpm = _soundData.bpm;
    }

    private function _timerInterruptionHandler():void
    {
        _newsSourceData.updateBitmapData();
        if (_soundData.masterVolume > 0) soundOn();
    }


    private function soundOn():void
    {
        var i:int;
        for (i = 0; i < _bdH; i++)
        {
            var color:int = _newsSourceData.checkActiveSound(i);
            if (color > 0)
            {
                var volume:Number = _vol[i] - _noteOnPrevList[i] * (_vol[i]);
                volume =  _vol[i];
                _noteOnPrevList[i]++;
                if(_noteOnPrevList[i] > 4)
                {
                    _noteOnPrevList[i] = 0;
                }
                _trackUpper = _driver.noteOn(_soundData.notes[i], _voices[i], _length[i]);
                _trackUpper.masterVolume = volume;
            }
            else
            {
                _noteOnPrevList[i] = 0;
            }
        }
    }

    public function soundOff():void
    {
        var i:int;
        for (i = 0; i < _bdH; i++)
        {
            _driver.noteOff(_soundData.notes[i]);
        }
    }


    private function _beatHandler(e:Event):void
    {
        _main3DViewController.viewUpdate();

        _soundData.beatCount++;
        if (_soundData.beatTrigger && _soundData.isTitleAnimeComplete)
        {
            _main3DViewController.rhythmEffect();
        }
    }
}
}





