/**
 * Created by 393 on 2014/05/30.
 */
package model
{
import flash.events.Event;
import flash.events.EventDispatcher;
import flash.events.IEventDispatcher;
import flash.media.SoundMixer;
import flash.media.SoundTransform;
import flash.utils.Dictionary;

import org.si.sion.SiONVoice;
import org.si.sion.utils.SiONPresetVoice;

public class SoundData extends EventDispatcher
{
    public static const BEAT_RHYTHM:String = "beatRythm";
    public static const DRIVER_PLAY_CHANGE:String = "driverPlayChange";

    private const BEAT_RATIO:int = 8;
    private const DEFAULT_BPM:int = 120;
    public static const DRUM_VOLUME:Number = 0.4;
    public static const DRIVER_VOLUME:Number = 0.5;

    private var _notesKey:String = "A";
    private var _notesG:Vector.<int> = Vector.<int>([1, 1, 1, 1, 1, 60, 62, 67, 72, 74, 76, 79 - 12, 1]);
    private var _notesF:Vector.<int> = Vector.<int>([1, 1, 1, 1, 1, 59, 60, 65, 71, 72, 74, 77 - 12, 1]);
    private var _notesE:Vector.<int> = Vector.<int>([1, 1, 1, 1, 1, 57, 59, 64, 69, 71, 72, 76, 1]);
    private var _notesD:Vector.<int> = Vector.<int>([1, 1, 1, 1, 1, 55, 57, 62, 67, 69, 71, 74, 1]);
    private var _notesC:Vector.<int> = Vector.<int>([1, 1, 1, 1, 1, 53, 55, 60, 65, 67, 69, 72, 1]);
    private var _notesB:Vector.<int> = Vector.<int>([1, 1, 1, 1, 1, 64 - 12, 65 - 12, 71 - 12, 76 - 12, 77 - 12, 79 - 12, 83 - 12, 1]);
    private var _notesA:Vector.<int> = Vector.<int>([1, 1, 1, 1, 1, 62 - 12, 64 - 12, 69 - 12, 74 - 12, 76 - 12, 77 - 12, 81 - 12, 1]);

    private var _notesList:Dictionary = new Dictionary();
    private var _notes:Vector.<int>;
    private var _length:Vector.<int> = Vector.<int>([ 1, 2, 4, 2, 4, 3, 3, 3, 3, 3, 3, 3, 1]);
    private var _vol:Vector.<int> = Vector.<int>([0, 10, 20, 20, 20, 5, 5, 5, 5, 10, 10, 10, 5]);
    private var _voices:Vector.<SiONVoice> = new Vector.<SiONVoice>();

    private var _beatCount:int;
    private var _beatTrigger:Boolean;

    private var _bpm:int = DEFAULT_BPM;

    private var _masterVolume:Number = 1;
    private var _masterSoundTransform:SoundTransform = new SoundTransform();

    private var _isDriverPlay:Boolean;
    private var _isTitleAnimeComplete:Boolean = false;
    private var _presetVoice:SiONPresetVoice;

    public function SoundData(target:IEventDispatcher = null)
    {
        super(target);
        initNotes();
        initVoice();
    }

    private function initVoice():void
    {
        _presetVoice = new SiONPresetVoice();
        _voices[0] = _presetVoice["valsound.percus22"];
        _voices[1] = _presetVoice["valsound.percus19"];
        _voices[2] = _presetVoice["valsound.percus27"];
        _voices[3] = _presetVoice["valsound.percus33"];
        _voices[4] = _presetVoice["valsound.percus3"];

        var i:int;
        for (i = 5; i < 9; i++)
        {
            _voices[i] = _presetVoice["konami"];
        }
        for (i = 9; i < 13; i++)
        {
            _voices[i] = _presetVoice["konami"];
        }
    }

    private function initNotes():void
    {
        _notesList["A"] = _notesA;
        _notesList["B"] = _notesB;
        _notesList["C"] = _notesC;
        _notesList["D"] = _notesD;
        _notesList["E"] = _notesE;
        _notesList["F"] = _notesF;
        _notesList["G"] = _notesG;
        _notes = _notesA;
    }

    public function get beatCount():int
    {
        return _beatCount;
    }

    public function set beatCount(value:int):void
    {
        _beatCount = value;

        _beatTrigger = false;
        if (_beatCount % BEAT_RATIO == 0)
        {
            _beatTrigger = true;
            dispatchEvent(new Event(SoundData.BEAT_RHYTHM))
        }
    }

    public function get beatTrigger():Boolean
    {
        return _beatTrigger;
    }

    public function get bpm():int
    {
        return _bpm;
    }

    public function set bpm(value:int):void
    {
        _bpm = value;
        dispatchEvent(new Event(Event.CHANGE));
    }

    public function get notes():Vector.<int>
    {
        return _notes;
    }

    public function get masterVolume():Number
    {
        return _masterVolume;
    }

    public function set masterVolume(value:Number):void
    {
        if (_masterVolume == value) return;

        _masterSoundTransform.volume = _masterVolume = value;
        SoundMixer.soundTransform = _masterSoundTransform;
    }

    public function set notesKey(value:String):void
    {
        if (!_notesList[value]) value = "A";
        _notes = _notesList[value];
        _notesKey = value;
    }

    public function setVoice(voice:String):void
    {
        var i:int;
        for (i = 5; i < 13; i++)
        {
            _voices[i] = _presetVoice[voice];
        }
    }

    public function get voices():Vector.<SiONVoice>
    {
        return _voices;
    }

    public function set voices(value:Vector.<SiONVoice>):void
    {
        _voices = value;
    }

    public function get length():Vector.<int>
    {
        return _length;
    }

    public function get vol():Vector.<int>
    {
        return _vol;
    }

    public function get isDriverPlay():Boolean
    {
        return _isDriverPlay;
    }

    public function set isDriverPlay(value:Boolean):void
    {
        _isDriverPlay = value;
        dispatchEvent(new Event(SoundData.DRIVER_PLAY_CHANGE))
    }

    public function get isTitleAnimeComplete():Boolean
    {
        return _isTitleAnimeComplete;
    }

    public function set isTitleAnimeComplete(value:Boolean):void
    {
        _isTitleAnimeComplete = value;
    }

}
}
