/**
 * Created by 393 on 2014/06/04.
 */
package view_controller
{
import flash.geom.Vector3D;

import model.BasicData;
import model.NewsSourceData;

public class CubePositionController
{
    private var _bdW:int = BasicData.TEXT_DISPLAY_WIDTH;
    private var _bdH:int = BasicData.TEXT_DISPLAY_HEIGHT;
    private var _cubeSize:int = BasicData.CUBE_SIZE;
    private var _offsetPositionRatio:Number = BasicData.CUBE_POSITION_OFFSET_RATIO;
    private var _cubeContainer:CubeContainer;
    private var _newsSourceData:NewsSourceData;

    public function CubePositionController(newsSourceData:NewsSourceData, cubeContainer:CubeContainer)
    {
        _cubeContainer = cubeContainer;
        _newsSourceData = newsSourceData;
        init();
    }

    private function init():void
    {
        initSpherePosition(false);
        initSpherePosition(true);
        initDefaultPosition(false);
        initDefaultPosition(true);
        initCapLessBoxPosition(false);
        initCapLessBoxPosition(true);
        initCrossPosition(false);
        initCrossPosition(true);
        initBoxPosition(false);
        initBoxPosition(true);
        initPyramidPosition(false);
        initPyramidPosition(true);
        initPipePosition(false);
        initPipePosition(true);
        initRandomPosition();
    }

    private function initDefaultPosition(isNoSpace:Boolean = false):void
    {
        var px:int, py:int, pz:int;

        var distance:Number;
        if (isNoSpace) distance = _cubeSize;
        else distance = _cubeSize * _offsetPositionRatio;
        for (var i:int = 0; i < _bdH; i++)
        {
            for (var j:int = 0; j < _bdW; j++)
            {
                px = j * distance - (_bdW * distance / 2 - _cubeSize * .8);
                py = i * distance - (_bdH * distance / 2 - _cubeSize * .8);
                pz = 0;
                if (isNoSpace) _cubeContainer.cubeList[i][j].defaultPositionNoSpace = new Vector3D(px, py, pz)
                else _cubeContainer.cubeList[i][j].defaultPosition = new Vector3D(px, py, pz)
            }
        }
    }

    private function initSpherePosition(isNoSpace:Boolean = false):void
    {
        var radian:Number = Math.PI / 180;
        var wRatio:Number = 360 / _bdW * radian;
        var hRatio:Number = 360 / _bdH * radian;
        var radius:int
        if (isNoSpace) radius = 50;
        else radius = 110;
        var px:int, py:int, pz:int;
        for (var i:int = 0; i < _bdH; i++)
        {
            for (var j:int = 0; j < _bdW; j++)
            {
                px = radius * Math.sin(i * hRatio / 2) * Math.cos(j * wRatio + Math.PI / 2 + wRatio * .6);
                py = radius * -Math.cos(i * hRatio / 2);
                pz = radius * Math.sin(i * hRatio / 2) * Math.sin(j * wRatio + Math.PI / 2 + wRatio * .6);
                if (isNoSpace) _cubeContainer.cubeList[i][j].spherePositionNoSpace = new Vector3D(px, py, pz)
                else _cubeContainer.cubeList[i][j].spherePosition = new Vector3D(px, py, pz)
            }
        }
    }

    private function initPipePosition(isNoSpace:Boolean = false):void
    {
        var radian:Number = Math.PI / 180;
        var wRatio:Number = 360 / _bdW * radian;
        var hRatio:Number = 360 / _bdH * radian;
        var radius:int;
        if (isNoSpace) radius = 50;
        else radius = 110;
        var px:int, py:int, pz:int;
        for (var i:int = 0; i < _bdH; i++)
        {
            for (var j:int = 0; j < _bdW; j++)
            {
                px = radius * Math.cos(j * wRatio + Math.PI / 2 + wRatio * .6);
                py = radius * -Math.cos(i * hRatio / 2);
                pz = radius * Math.sin(j * wRatio + Math.PI / 2 + wRatio * .6);
                if (isNoSpace) _cubeContainer.cubeList[i][j].pipePositionNoSpace = new Vector3D(px, py, pz)
                else _cubeContainer.cubeList[i][j].pipePosition = new Vector3D(px, py, pz)
            }
        }
    }

    private function initBoxPosition(isNoSpace:Boolean = false):void
    {
        var count:int = 0;
        var wRatio:int = 6;
        var hRatio:int = wRatio * wRatio;
        var distance:Number
        if (isNoSpace) distance = _cubeSize;
        else distance = _cubeSize * _offsetPositionRatio;
        var px:int, py:int, pz:int;
        for (var i:int = 0; i < _bdH; i++)
        {
            for (var j:int = 0; j < _bdW; j++)
            {
                px = (count % wRatio) * distance - (wRatio * distance / 2 - _cubeSize * .8);
                pz = int(count / wRatio) % wRatio * distance - (wRatio * distance / 2 - _cubeSize * .8);
                var yRatio:int = int(count / hRatio);
                if (yRatio > wRatio) yRatio = wRatio;
                py = yRatio * distance - (wRatio * distance / 2 - _cubeSize * .8);
                if (isNoSpace) _cubeContainer.cubeList[i][j].boxPositionNoSpace = new Vector3D(px, py, pz)
                else _cubeContainer.cubeList[i][j].boxPosition = new Vector3D(px, py, pz)
                count++;
            }
        }
    }

    private function initCapLessBoxPosition(isNoSpace:Boolean = false):void
    {
        var wRatio:int = 6;
        var px:int, py:int, pz:int, jx:int, jz:int, ratio:Number;

        if (isNoSpace) ratio = 1;
        else ratio = 2;
        var length:int = int(_bdW / 4);

        for (var i:int = 0; i < _bdH; i++)
        {
            for (var j:int = 0; j < _bdW - 1; j++)
            {
                if (j <= length)
                {
                    jx = j;
                    jz = 0;
                }
                else if (j > length && j <= length * 2 - 1)
                {
                    jx = length;
                    jz = j - length;
                }
                else if (j > length * 2 - 1 && j <= length * 3 - 1)
                {
                    jx = length * 3 - j;
                    jz = length;
                }
                else
                {
                    jx = 0;
                    jz = length * 4 - j;
                }
                px = jx * _cubeSize * ratio - (wRatio * _cubeSize * ratio / 2 - _cubeSize * .8);
                pz = jz * _cubeSize * ratio - (wRatio * _cubeSize * ratio / 2 - _cubeSize * .8);
                py = i * _cubeSize * ratio - (_bdH * _cubeSize * ratio / 2 - _cubeSize * .8);
                if (isNoSpace) _cubeContainer.cubeList[i][j].capLessBoxPositionNoSpace = new Vector3D(px, py, pz)
                else _cubeContainer.cubeList[i][j].capLessBoxPosition = new Vector3D(px, py, pz);
            }
        }
    }

    private function initCrossPosition(isNoSpace:Boolean = false):void
    {
        var px:int, py:int, pz:int, jx:int, jz:int, ratio:Number;

        if (isNoSpace) ratio = 1;
        else ratio = 2;
        var length:int = int(_bdW / 4);

        for (var i:int = 0; i < _bdH; i++)
        {
            for (var j:int = 0; j < _bdW; j++)
            {
                jx = jz = 0;
                if (j <= length)
                {
                    jx = j % length;
                }
                else if (j > length && j <= length * 2 - 1)
                {
                    jz = j % length;
                }
                else if (j > length * 2 - 1 && j <= length * 3 - 1)
                {
                    jx = -j % length;
                }
                else
                {
                    jz = -j % length;
                }
                px = jx * _cubeSize * ratio;
                pz = jz * _cubeSize * ratio;
                py = i * _cubeSize * ratio - (_bdH * _cubeSize * ratio / 2 - _cubeSize * .8);
                if (isNoSpace) _cubeContainer.cubeList[i][j].crossPositionNoSpace = new Vector3D(px, py, pz);
                else _cubeContainer.cubeList[i][j].crossPosition = new Vector3D(px, py, pz);
            }
        }
    }

    private function initRandomPosition():void
    {
        var i:int , j:int, px:int, py:int, pz:int, distance:Number;
        distance = _cubeSize * _offsetPositionRatio;
        var wRatio:int = 400;
        var hRatio:int = 400;
        for (i = 0; i < _bdH; i++)
        {
            for (j = 0; j < _bdW; j++)
            {
                var r0:Number = Math.random() - 0.5;
                var r1:Number = Math.random() - 0.5;
                var r2:Number = Math.random() - 0.5;
                px = j * distance - (_bdW * distance / 2 - _cubeSize * .8) + r0 * wRatio;
                py = i * distance - (_bdH * distance / 2 - _cubeSize * .8) + r1 * hRatio;
                pz = r2 * hRatio;
                _cubeContainer.cubeList[i][j].randomPosition = new Vector3D(px, py, pz)
            }
        }
    }

    private function initPyramidPosition(isNoSpace:Boolean = false):void
    {
        var i:int, j:int, length:int, px:int, py:int, pz:int, distance:Number, offsetYLength:int = 5;
        var cubeCount:int = 0;
        var yCount:int = 0;
        if (isNoSpace) distance = _cubeSize;
        else distance = _cubeSize * _offsetPositionRatio;

        length = _bdH * _bdW;

        for (i = 1; i < length; i++)
        {
            var m:int = i * i;
            for (j = 0; j < m; j++)
            {
                if (i != 9)
                {
                    px = j % i * distance - yCount * distance / 2;
                    pz = int(j / i) * distance - yCount * distance / 2;
                    py = -yCount * distance + offsetYLength * distance;
                }
                if (isNoSpace) _cubeContainer.cubeListSingle[cubeCount].pyramidPositionNoScale = new Vector3D(px, py, pz);
                else _cubeContainer.cubeListSingle[cubeCount].pyramidPosition = new Vector3D(px, py, pz);
                cubeCount++;
                if (cubeCount > length - 1) break;
            }
            yCount++;
            if (cubeCount > length - 1) break;
        }
    }

    private function initWavePosition(soundGain:Number):void
    {
        var ratio:int = soundGain * 4;
        var wRatio:Number = Math.PI * ratio / _bdW;
        var hRatio:Number = Math.PI * ratio / _bdH;

        var count:int = Math.PI * 2 * Math.random();
        var yGain:Number = soundGain * 30;

        var i:int , j:int, px:int, py:int, pz:int;
        for (i = 0; i < _bdH; i++)
        {
            for (j = 0; j < _bdW; j++)
            {
                px = Math.sin(i * wRatio + count) * yGain + _cubeContainer.cubeList[i][j].defaultPosition.x;
                py = Math.sin(j * hRatio + count) * yGain + _cubeContainer.cubeList[i][j].defaultPosition.y;
                pz = 0;
                _cubeContainer.cubeList[i][j].wavePosition = new Vector3D(px, py, pz)
            }
        }
    }


    public function setDefaultPosition(isNoSpace:Boolean = false,isNoDelay:Boolean = false):void
    {
        var i:int, j:int;
        for (i = 0; i < _bdH; i++)
        {
            for (j = 0; j < _bdW; j++)
            {
                _cubeContainer.cubeList[i][j].setDefaultPosition(isNoSpace,isNoDelay);
            }
        }
    }

    public function setBoxPosition(isNoSpace:Boolean = false,isNoDelay:Boolean = false):void
    {
        var i:int, j:int;
        for (i = 0; i < _bdH; i++)
        {
            for (j = 0; j < _bdW; j++)
            {
                _cubeContainer.cubeList[i][j].setBoxPosition(isNoSpace,isNoDelay);
            }
        }
    }

    public function setCapLessBoxPosition(isNoSpace:Boolean = false,isNoDelay:Boolean = false):void
    {
        var i:int, j:int;
        for (i = 0; i < _bdH; i++)
        {
            for (j = 0; j < _bdW; j++)
            {
                _cubeContainer.cubeList[i][j].setCapLessBoxPosition(isNoSpace,isNoDelay);
            }
        }
    }

    public function setCrossPosition(isNoSpace:Boolean = false,isNoDelay:Boolean = false):void
    {
        var i:int, j:int;
        for (i = 0; i < _bdH; i++)
        {
            for (j = 0; j < _bdW; j++)
            {
                _cubeContainer.cubeList[i][j].setCrossPosition(isNoSpace,isNoDelay);
            }
        }
    }

    public function setSpherePosition(isNoSpace:Boolean = false,isNoDelay:Boolean = false):void
    {
        var i:int, j:int;
        for (i = 0; i < _bdH; i++)
        {
            for (j = 0; j < _bdW; j++)
            {
                _cubeContainer.cubeList[i][j].setSpherePosition(isNoSpace,isNoDelay);
            }
        }
    }

    public function setPipePosition(isNoSpace:Boolean = false,isNoDelay:Boolean = false):void
    {
        var i:int, j:int;
        for (i = 0; i < _bdH; i++)
        {
            for (j = 0; j < _bdW; j++)
            {
                _cubeContainer.cubeList[i][j].setPipePosition(isNoSpace,isNoDelay);
            }
        }
    }

    public function setRandomPosition(isNoSpace:Boolean = false,isNoDelay:Boolean = false):void
    {
        var i:int, j:int;
        for (i = 0; i < _bdH; i++)
        {
            for (j = 0; j < _bdW; j++)
            {
                _cubeContainer.cubeList[i][j].setRandomPosition(isNoDelay);
            }
        }
    }

    public function setPyramidPosition(isNoSpace:Boolean = false,isNoDelay:Boolean = false):void
    {
        var i:int, j:int;
        for (i = 0; i < _bdH; i++)
        {
            for (j = 0; j < _bdW; j++)
            {
                _cubeContainer.cubeList[i][j].setPyramidPosition(isNoSpace,isNoDelay);
            }
        }
    }

    public function setRandom():void
    {
        var isSpace:Boolean = true;
        var funcArray:Array = [
            setPyramidPosition
           ,setBoxPosition
           ,setSpherePosition
           ,setCapLessBoxPosition
           ,setCrossPosition
           ,setRandomPosition
           ,setPipePosition
        ];
        var randamIndex:int = Math.random() * funcArray.length;
        if(Math.random() > 0.5) isSpace = false;
        funcArray[randamIndex](isSpace);
    }

    public function setWave(soundGain:Number):void
    {
        initWavePosition(soundGain);
        var i:int, j:int;
        for (i = 0; i < _bdH; i++)
        {
            for (j = 0; j < _bdW; j++)
            {
                _cubeContainer.cubeList[i][j].setWave();
            }
        }
    }
}
}
