/**
 * Created by 393 on 2014/05/29.
 */
package view_controller
{
import a24.tween.Tween24;

import away3d.cameras.lenses.PerspectiveLens;
import away3d.containers.View3D;
import away3d.controllers.HoverController;
import away3d.filters.BloomFilter3D;
import away3d.filters.RadialBlurFilter3D;
import away3d.lights.DirectionalLight;
import away3d.materials.lightpickers.StaticLightPicker;

import controller.SoundController;

import flash.display.Sprite;
import flash.events.Event;
import flash.events.MouseEvent;
import flash.geom.Point;
import flash.ui.Mouse;
import flash.ui.MouseCursor;

import model.BasicData;
import model.MaterialData;
import model.NewsSourceData;
import model.SoundData;

public class Main3DViewController extends ViewController
{
    private var _bdW:int = BasicData.TEXT_DISPLAY_WIDTH;
    private var _bdH:int = BasicData.TEXT_DISPLAY_HEIGHT;

    private var _soundController:SoundController;
    private var _cubePositionController:CubePositionController;
    private var _cameraController:HoverController;

    private var _newsSourceData:NewsSourceData;
    private var _soundData:SoundData;

    private var _view:View3D;
    private var _radialBlurFilter3D:RadialBlurFilter3D;
    private var _bloomFilter3D:BloomFilter3D;
    private var _staticLightPicker:StaticLightPicker;
    private var _cubeContainer:CubeContainer;
    private var _lightList:Array;
    private var _cameraSpeed:Number = -.3;

    private var _isChangeTextTransitionPlaying:Boolean;
    private var _panAngle:Number = 0;
    private var _tiltAngle:Number = 0;
    private var _mousePoint:Point = new Point();
    private var _isDragging:Boolean;

    public function Main3DViewController(newsSourceData:NewsSourceData, soundData:SoundData)
    {
        super();
        _newsSourceData = newsSourceData;
        _soundData = soundData;
        view = new Sprite();
        view.addEventListener(Event.ADDED_TO_STAGE, init);
    }


    private function init(event:Event = null):void
    {
        _soundController = new SoundController(_newsSourceData, _soundData, this);

        view.addEventListener(MouseEvent.ROLL_OVER, function(event:MouseEvent):void{Mouse.cursor = MouseCursor.HAND})
        view.addEventListener(MouseEvent.ROLL_OUT, function(event:MouseEvent):void{Mouse.cursor = MouseCursor.BUTTON})

        init3D();
        initFilter();
        initLight();
        initMaterial();
        initCube();

        _newsSourceData.setTitleData();
    }


    private function init3D():void
    {
        _view = new View3D();
        _view.width = BasicData.STAGE_WIDTH;
        _view.height = BasicData.STAGE_HEIGHT;
        view.addChild(_view);
        _view.camera.lens = new PerspectiveLens(60);
        _cameraController = new HoverController(_view.camera, null, 0, 0, -100);
        _cameraController.distance = -240;
        _cameraController.steps = 2;

		//view.addChild(new AwayStats(_view));
    }

    private function initCube():void
    {
        _cubeContainer = new CubeContainer(_newsSourceData);
        _cubePositionController = new CubePositionController(_newsSourceData, _cubeContainer);
        _view.scene.addChild(_cubeContainer);
    }

    private function initLight():void
    {
        _lightList = [];
        _lightList[0] = new DirectionalLight(-0, -0.1, .5);
        _lightList[1] = new DirectionalLight(-0, -0.1, -.5);
        _lightList[0].ambient = 0.2;
        _lightList[1].ambient = 0;
        _lightList[0].ambientColor = 0xFFFFFF;
        _lightList[1].ambientColor = 0xFFFFFF;
        _staticLightPicker = new StaticLightPicker(_lightList);

    }

    private function initMaterial():void
    {
        MaterialData.init(_staticLightPicker);
    }

    private function initFilter():void
    {
        _radialBlurFilter3D = new RadialBlurFilter3D(0, 2, 1, -0.4, 0.5, 0.5);
        _bloomFilter3D = new BloomFilter3D(14, 14, 0.75, 3);
        _view.filters3d = [_radialBlurFilter3D, _bloomFilter3D];
    }

    public function renderReady():void
    {
        openingAnimeStart();
        view.addEventListener(Event.ENTER_FRAME, enterFrameHandler);
        _view.render();
    }

    private function openingAnimeStart():void
    {
        _cubePositionController.setDefaultPosition(true);
        _newsSourceData.setBitmapData();
        analyzeNewsData(0);


        Tween24.func(_cubePositionController.setDefaultPosition,false,true).delay(2).play();
        Tween24.func(function():void{
            _soundData.isDriverPlay = true;
        }).delay(2.5).play();

        view.stage.addEventListener(MouseEvent.MOUSE_DOWN,downHandler);
    }

    private function downHandler(event:MouseEvent):void
    {
        view.stage.removeEventListener(MouseEvent.MOUSE_DOWN,downHandler);

        view.stage.addEventListener(MouseEvent.MOUSE_UP,upHandler);
        view.stage.addEventListener(MouseEvent.MOUSE_OUT,outHandler);
        view.stage.addEventListener(Event.MOUSE_LEAVE, leaveHandler);
        _isDragging = true;
        _panAngle = _cameraController.panAngle;
        _tiltAngle = _cameraController.tiltAngle;
        _mousePoint.x = view.stage.mouseX;
        _mousePoint.y = view.stage.mouseY;
        _cameraSpeed = .5;
    }

    private function outHandler(event:MouseEvent):void
    {
        var px:Number = event.stageX;
        var py:Number = event.stageY;
        if (px <=0 || px >= view.stage.stageWidth || py <= 0 || py >= view.stage.stageHeight) {
            upHandler(event);
        }
    }


    private function leaveHandler(event:Event):void
    {
        view.stage.removeEventListener(Event.MOUSE_LEAVE, leaveHandler);
        view.stage.removeEventListener(MouseEvent.MOUSE_OUT,outHandler);
        _isDragging = false;
        _cameraSpeed = -.3;
    }

    private function upHandler(event:MouseEvent):void
    {
        view.stage.removeEventListener(MouseEvent.MOUSE_UP,upHandler);
        view.stage.addEventListener(MouseEvent.MOUSE_DOWN,downHandler);
        _isDragging = false;
        _cameraSpeed = -.3;
    }

    private function enterFrameHandler(event:Event):void
    {
        filterEffectUpdate();
        if (_newsSourceData.isTitleTextFinished && _soundData.isDriverPlay ) _view.camera.z += _cameraSpeed;

        if (_isDragging) {
            _cameraController.panAngle = _panAngle + (view.stage.mouseX - _mousePoint.x)*.5
            _cameraController.tiltAngle = _tiltAngle + (view.stage.mouseY - _mousePoint.y)*.2
        }
        _view.render();
    }

    private function filterEffectUpdate():void
    {
        if (_radialBlurFilter3D.intensity >= 0)
        {
            _radialBlurFilter3D.intensity -= .4;
        }
        else
        {
            _radialBlurFilter3D.intensity = 0;
        }
        if (_bloomFilter3D.exposure >= 3)
        {
            _bloomFilter3D.exposure -= 0.4
        }
        else
        {
            _bloomFilter3D.exposure = 3;
        }
    }

    public function effectUpdate(soundGain:Number = 0):void
    {
        if (_newsSourceData.isTitleTextFinished || _soundData.isTitleAnimeComplete)
        {
            _radialBlurFilter3D.intensity = soundGain * 2.5;
            _bloomFilter3D.exposure = soundGain * 3;

            if (soundGain > 0.9)
            {
                changeCameraAngle();
                _cubeContainer.beatEffect();
            }
        }
    }

    private function changeCameraAngle(tweenTime:Number = 0.3):void
    {
        _cameraController.tiltAngle = (Math.random()) * 10;
        _cameraController.panAngle = (Math.random() - 0.5) * 35;
        var dist:Number = int(Math.random() * 3) * -50 - 220;
        Tween24.tween(_cameraController,tweenTime,Tween24.ease.QuadOut, {distance: dist}).play();
    }

    public function viewUpdate():void
    {
        if (!_newsSourceData.sourceTextFieldBitmapData && ! _soundData.isTitleAnimeComplete || _isChangeTextTransitionPlaying)
        {
            return;
        }
        var soundGain:Number = getSoundGain();
        var isCurrentTextFinished:Boolean = analyzeNewsData(soundGain);
        effectUpdate(soundGain);
        checkNewsScrollFinished(isCurrentTextFinished);
    }

    private function checkNewsScrollFinished(isCurrentTextFinished:Boolean):void
    {
        if (isCurrentTextFinished && _newsSourceData.textFieldWidthCount < 0)
        {
            if (!_newsSourceData.isTitleTextFinished)
            {
                _soundData.isTitleAnimeComplete = true;
                _newsSourceData.setLoadData2SourceNewsList();
                changeTextTransition(false);
            }
            else
            {
                updateNextText();
                changeTextTransition(true);
            }
        }
    }

    public function updateNextText():void
    {
        _newsSourceData.sourceTextFieldBitmapData.dispose();
        _newsSourceData.currentIndex++;
    }

    private function analyzeNewsData(soundGain:Number):Boolean
    {
        var i:int, j:int, i2:int, isFinished:Boolean = true;
        var bmdColorList:Array = [];

        _newsSourceData.sourceTextFieldBitmapData.lock();
        for (i = 0; i < _bdH; i++)
        {
            bmdColorList[i] = [];

            i2 = _bdH - i - 1;
            for (j = 0; j < _bdW; j++)
            {
                var pixelColor:int = _newsSourceData.sourceTextFieldBitmapData.getPixel(j, i);
                if ((_soundData.isTitleAnimeComplete || _newsSourceData.isTitleTextFinished) && BasicData.SOUND_ON_POSITION_X_LIST[i] == j)
                {
                    if (pixelColor == BasicData.TEXT_FORMAT_COLOR)
                    {
                        _cubeContainer.cubeList[i2][j].changeYellowColor(soundGain);
                        isFinished = false;
                    }
                    else if (pixelColor == BasicData.TEXT_FORMAT_KEYWORD_COLOR)
                    {
                        _cubeContainer.cubeList[i2][j].changeKeywordColor();
                        _cubePositionController.setWave(soundGain * 3);
                        isFinished = false;
                    }
                    else
                    {
                        _cubeContainer.cubeList[i2][j].changeBlackColor();
                    }
                }
                else
                {
                    if (pixelColor == BasicData.TEXT_FORMAT_COLOR)
                    {
                        _cubeContainer.cubeList[i2][j].changeColor(soundGain, _newsSourceData.currentNewsColor);
                        isFinished = false;
                    }
                    else if (pixelColor == BasicData.TEXT_FORMAT_KEYWORD_COLOR)
                    {
                        _cubeContainer.cubeList[i2][j].changeKeywordColor();
                        isFinished = false;
                    }
                    else
                    {
                        _cubeContainer.cubeList[i2][j].changeBlackColor();
                    }
                }
            }
        }
        _newsSourceData.sourceTextFieldBitmapData.unlock();
        return isFinished;
    }

    public function changeTextTransition(isPositionRandom:Boolean = true):void
    {
        _isChangeTextTransitionPlaying = true;
        _cubeContainer.changeAllColor(_newsSourceData.currentNewsColor);

        Tween24.serial(
            Tween24.prop(_cubeContainer).rotationY(720)
            ,Tween24.tween(_cubeContainer,1.5,Tween24.ease.CircInOut).rotationY(0)
        ).delay(.5).onComplete(function():void{
            _isChangeTextTransitionPlaying = false
        }).play();

        if(isPositionRandom)
        {
            _cubePositionController.setRandom();
        }
        else
        {
            _cubePositionController.setRandomPosition();
        }

        changeCameraAngle(2);

        Tween24.func(function():void{
            _cubePositionController.setDefaultPosition();
            _cubeContainer.changeAllColor(0, 0.002);
        }).delay(1.3).play();
    }

    private function getSoundGain():Number
    {
        if (! _newsSourceData.isTitleTextFinished && ! _soundData.isTitleAnimeComplete)
        {
            return 0;
        }

        var i:int, soundGain:Number = 0;
        for (i = 0; i < _bdH-1; i++)
        {
            if (_newsSourceData.sourceTextFieldBitmapData.getPixel(BasicData.SOUND_ON_POSITION_X_LIST[i], i) > 0)
            {
                soundGain++;
            }
        }
        return soundGain / (_bdH - 1);
    }

    public function rhythmEffect():void
    {
        _cubeContainer.rhythmEffect();
    }

    public function get cubePositionController():CubePositionController
    {
        return _cubePositionController;
    }

    public function get cameraController():HoverController
    {
        return _cameraController;
    }
}
}
