/**
 * Created by 393 on 2014/05/29.
 */
package view_controller
{
import com.bit101.components.CheckBox;
import com.bit101.components.ComboBox;
import com.bit101.components.HUISlider;
import com.bit101.components.Panel;
import com.bit101.components.PushButton;

import flash.display.Sprite;
import flash.events.Event;
import flash.events.MouseEvent;

import model.BasicData;
import model.NewsSourceData;
import model.SoundData;

import org.si.sion.SiONVoice;
import org.si.sion.utils.SiONPresetVoice;

public class UIViewController extends ViewController
{
    private var _newsSourceData:NewsSourceData;
    private var _soundData:SoundData;
    private var _main3DViewController:Main3DViewController;

    public function UIViewController(newsSourceData:NewsSourceData, soundData:SoundData, main3DViewController:Main3DViewController)
    {
        super();
        _newsSourceData = newsSourceData;
        _soundData = soundData;
        _main3DViewController = main3DViewController;
        init();
    }

    private function init():void
    {
        view = new Sprite();

        var panel:Panel = new Panel(view, 10, 0);
        panel.width = 330;
        panel.height = 220;
        panel.y = 250;

        var bpmSlider:HUISlider = new HUISlider(panel, 5, 5, "BPM", bpmChangeHandler);
        bpmSlider.maximum = 240;
        bpmSlider.minimum = 40;
        bpmSlider.value = 120;

        var hitPositionRandom:CheckBox = new CheckBox(panel, 5, 25, "hit_position_random", hitPositionRandomChangeHandler);
        hitPositionRandom.selected = false;

        var soundKeyComboBox:ComboBox = new ComboBox(panel, 5, 45, "select Key", ["A", "B", "C", "D", "E", "F", "G"]);
        soundKeyComboBox.addEventListener(Event.SELECT, soundKeyChangeHandler);

        var voices:Array = [];
        var v:SiONPresetVoice = new SiONPresetVoice();
        for (var s:Object in v)
        {
            if (v[s] is SiONVoice) voices.push(s);
        }
        voices.sort();
        var soundVoiceComboBox:ComboBox = new ComboBox(panel, 125, 45, "select VOICE", voices);
        soundVoiceComboBox.addEventListener(Event.SELECT, soundVoiceChangeHandler);

        new PushButton(panel, 5, 65+20, "pose:default", defClickHandler);
        new PushButton(panel, 5, 85+20, "pose:sphere", sphrClickHandler);
        new PushButton(panel, 5, 105+20, "pose:capless box", capLessBoxClickHandler);
        new PushButton(panel, 5, 125+20, "pose:box", boxClickHandler);
        new PushButton(panel, 5, 145+20, "pose:pipe", pipeClickHandler);

        new PushButton(panel, 125, 150, "wave effect", waveMoveClickHandler);
        new PushButton(panel, 125, 170, "camera move", cameraMoveClickHandler);
        new PushButton(panel, 125, 190, "change Text", changeTextClickHandler);

        var cameraZSlider:HUISlider = new HUISlider(panel, 125, 85, "camera dist", cameraDistHandler);
        cameraZSlider.maximum = -1;
        cameraZSlider.minimum = -500;
        cameraZSlider.value = 0;
        var cameraTiltAngleSlider:HUISlider = new HUISlider(panel, 125, cameraZSlider.y + 20, "tilt angle", cameraTiltAngleHandler);
        cameraTiltAngleSlider.maximum = 30;
        cameraTiltAngleSlider.minimum = -30;
        cameraTiltAngleSlider.value = 0;
        var cameraPanAngleSlider:HUISlider = new HUISlider(panel, 125, cameraTiltAngleSlider.y + 20, "pan angle", cameraPanAngleHandler);
        cameraPanAngleSlider.maximum = 45;
        cameraPanAngleSlider.minimum = -45;
        cameraPanAngleSlider.value = 0;
    }

    private function changeTextClickHandler(event:MouseEvent):void
    {
        _main3DViewController.updateNextText();
    }

    private function soundVoiceChangeHandler(event:Event):void
    {
        var box:ComboBox = event.currentTarget as ComboBox;
        var voises:SiONPresetVoice = new SiONPresetVoice();
        for (var i:int = 5; i < 13; i++)
        {
            _soundData.voices[i] = voises[box.selectedItem];
        }
    }

    private function waveMoveClickHandler(event:MouseEvent):void
    {
        _main3DViewController.cubePositionController.setWave(1);
    }

    private function cameraMoveClickHandler(event:MouseEvent):void
    {
        _main3DViewController.changeTextTransition();
    }

    private function cameraPanAngleHandler(event:Event):void
    {
        _main3DViewController.cameraController.panAngle = HUISlider(event.target).value;
    }

    private function cameraTiltAngleHandler(event:Event):void
    {
        _main3DViewController.cameraController.tiltAngle = HUISlider(event.target).value;
    }

    private function boxClickHandler(event:MouseEvent):void
    {
        _main3DViewController.cubePositionController.setBoxPosition();
    }

    private function cameraDistHandler(event:Event):void
    {
        _main3DViewController.cameraController.distance = HUISlider(event.target).value;
    }

    private function pipeClickHandler(event:MouseEvent):void
    {
        _main3DViewController.cubePositionController.setPipePosition();
    }

    private function capLessBoxClickHandler(event:MouseEvent):void
    {
        _main3DViewController.cubePositionController.setCapLessBoxPosition();
    }

    private function sphrClickHandler(event:MouseEvent):void
    {
        _main3DViewController.cubePositionController.setSpherePosition();
    }

    private function defClickHandler(event:MouseEvent):void
    {
        _main3DViewController.cubePositionController.setDefaultPosition();
    }

    private function soundKeyChangeHandler(event:Event):void
    {
        var selectIndex:int = ComboBox(event.target).selectedIndex;
        var array:Array = ["A", "B", "C", "D", "E", "F", "G"];
        _soundData.notesKey = array[selectIndex];
    }

    private function hitPositionRandomChangeHandler(event:Event):void
    {
        var isSelected:Boolean = CheckBox(event.target).selected;
        setHitPosition(isSelected);
    }

    private function setHitPosition(isRandom:Boolean):void
    {
        BasicData.SOUND_ON_POSITION_X_LIST = [];
        for (var i:int = 0, l:int = BasicData.TEXT_DISPLAY_HEIGHT; i < l; i++)
        {
            if(isRandom)
            {
                BasicData.SOUND_ON_POSITION_X_LIST[i] = int(BasicData.TEXT_DISPLAY_WIDTH * Math.random());
            }
            else
            {
                BasicData.SOUND_ON_POSITION_X_LIST[i] = BasicData.TEXT_DISPLAY_WIDTH_HALF;
            }
        }
    }

    private function bpmChangeHandler(event:Event):void
    {
        _soundData.bpm = HUISlider(event.target).value;

    }
}
}
